from setuptools import setup,find_packages


setup(
    name='myalgo',
    version='1.2.2',
    packages=find_packages(),
    url='',
    license='',
    author='nathaniel',
    author_email='348831271@qq.com',
    description='a simple trader for CTA trading.'
)
